/* Engine */
#include<stdio.h>
#include<stdlib.h>
#include "engine.h"

struct Game game;
static int dir_x[4]={-1,0,1,0},dir_y[4]={0,-1,0,1};

void new_game()
{
	game.level=1;
	game.lives=5;
	game.score=0;
	game.invalidate=0;
}

void new_level(int level)
{
	int i,j,x,y;

	srand(12345*level);
	for(i=0;i<MAPWIDTH*MAPHEIGHT;i++) game.map[i]=M_OPENPATH;
	// Now create the path sections
	for(y=3;y<MAPHEIGHT+1;y+=4) {
		for(x=0;x<MAPWIDTH;x++) {
			game.map[x+MAPWIDTH*y]=M_WALL;
		}
		for(i=0;i<level;i++) {
			x=rand()%(MAPWIDTH-9);
			x&=~1;
			x+=4;
			if(game.map[x+MAPWIDTH*(y-1)]!=M_LADDER) {
				game.map[x+MAPWIDTH*y]=M_HOLE;
			}
		}
		if(y&4) {
			// Left hand ladder
			x=(rand()%2);
		} else {
			// Right hand
			x=MAPWIDTH-1-(rand()%2);
		}
		for(j=0;j<4;j++) {
			if(y+j<MAPHEIGHT+1) game.map[x+MAPWIDTH*(y+j)]=M_LADDER;
		}
	}
	game.player.x=TILEWIDTH*0;
	game.player.y=TILEHEIGHT*2;
	game.player.jump=0;
	game.player.velocity=0;
	game.player.delay=TILEWIDTH;
	game.player.dir=D_RIGHT;
	game.invalidate=2;
//	if(((game.level/3)%2)==0) play_sound(S_RUSH);
//	else play_sound(S_ALONE);
}

void draw_map()
{
	int x,y,i=0;
	int top,bottom;
	top=game.player.y/TILEHEIGHT-1;
	bottom=game.player.y/TILEHEIGHT+2;
	for(y=0;y<MAPHEIGHT;y++) {
		if(game.invalidate || (y>=top && y<=bottom)) {
			for(x=0;x<MAPWIDTH;x++) {
				if(game.map[x+y*MAPWIDTH]==M_LADDER) {
					draw_tile(M_OPENPATH,x*TILEWIDTH,y*TILEHEIGHT);
				}
				draw_tile(game.map[x+y*MAPWIDTH],x*TILEWIDTH,y*TILEHEIGHT);
			}
		}
	}
	if(game.invalidate>0) game.invalidate--;
	draw_tile(M_PLAYER+((game.player.x&2)?1:0),game.player.x,game.player.y-game.player.jump/16);
	for(i=0;i<3;i++) {
		draw_tile(M_LEVEL+i,TILEWIDTH*i,0);
		draw_tile(M_SCORE+i,TILEWIDTH*(i+5),0);
	}
	draw_tile(M_WHITE,TILEWIDTH*12,0);
	draw_tile(M_WHITE,TILEWIDTH*13,0);
	//draw_tile(M_TREASURE,TILEWIDTH*13,0);
	draw_tile(M_WHITE,TILEWIDTH*16,0);
	draw_tile(M_WHITE,TILEWIDTH*17,0);
	draw_tile(M_PLAYER,TILEWIDTH*17,0);
	// Now decode the number into decimal.
	int num;
	num=game.level;
	draw_number((num/10)%10,TILEWIDTH*3,0);
	draw_number(num%10,TILEWIDTH*4,0);

	num=game.score;
	draw_number((num/1000)%10,TILEWIDTH*8,0);
	draw_number((num/100)%10,TILEWIDTH*9,0);
	draw_number((num/10)%10,TILEWIDTH*10,0);
	draw_number(num%10,TILEWIDTH*11,0);	

	num=game.lives;
	if(num<0) num=0;
	draw_number(num%10,TILEWIDTH*18,0);
	
	if(game.lives<1) {
		for(i=0;i<5;i++) {
			// Game Over
			draw_number(10+i,TILEWIDTH*(MAPWIDTH/2-2+i),TILEHEIGHT*5);
		}
	}
}

void move_character(int direction)
{
	// No effect
}

void explode_path()
{
	int x,y;

	if(game.lives==0) {
		new_game();
		new_level(game.level);
		return;
	}
	if(game.player.jump>0) return;
	game.player.jump=1;
	game.player.velocity=16;
	play_sound(S_BALLBAT);
}

void next_frame_tick();

void next_frame(int elapsed)
{
	while(elapsed>5) {
		next_frame_tick();
		elapsed-=5;
	}
//	next_frame_tick();
//	SDL_Delay(20);
}

void next_frame_tick()
{
	if(game.lives<1) return;

	// Now movement time
	if(game.player.dir>=0 && game.player.delay) {
		game.player.x+=dir_x[game.player.dir];
		game.player.y+=dir_y[game.player.dir];
	}
	if(game.player.jump==1 && game.player.velocity<0) {
		// Jump complete.
		game.player.jump=0;
		game.player.velocity=0;
	}
	if(game.player.jump>0) {
		game.player.jump+=game.player.velocity;
		game.player.velocity--;
	}

	if(game.player.delay>0)
		game.player.delay--;
//	if(game.player.delay==0) game.player.dir=D_NONE;
	
	if(game.player.y>TILEHEIGHT*(MAPHEIGHT-1)) {
		// finish level.
		game.level++;
		play_sound(S_APPLAUSE);
		new_level(game.level);
		return;
	}

	int x,y;
	
	if(game.player.delay>0) {
		return;
	}
	x=game.player.x;
	y=game.player.y;
	if(game.player.dir>=0 && game.player.delay) {
		x+=dir_x[game.player.dir]*game.player.delay;
		y+=dir_y[game.player.dir]*game.player.delay;
	}
	game.player.delay=TILEWIDTH;
	x/=TILEWIDTH;
	y/=TILEHEIGHT;
	y+=1;
//	game.map[y*MAPWIDTH+x]=M_VISITEDPATH;
	int cell=game.map[y*MAPWIDTH+x];
	switch(cell) {
		case M_WALL:
			if(game.player.dir==D_DOWN && game.map[(y-1)*MAPWIDTH+x]==M_LADDER) {
				// Just went down the ladder, so continue running
				game.player.dir=x>MAPWIDTH/2?D_LEFT:D_RIGHT;
			} else if(game.player.dir==D_DOWN) {
				// Crash!
				play_sound(S_BOMB);
				game.lives--;
				new_level(game.level);	// Same level reset.
				return;
			}
			break;
		case M_HOLE:
			if(game.player.jump>0) break;
		case M_OPENPATH:
		case M_LADDER:
			game.player.dir=D_DOWN;
			break;
	}
//	play_sound(S_BALLBAT);
}
