#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include "SDL_mixer.h"

#ifdef _WIN32
#include <windows.h>
#endif
#include "engine.h"

/*
 PLEASE NOTE: the program will require SDL.dll which is located in
              dev-c++'s dll directory. You have to copy it to you
			  program's home directory or the path.
 */

/* The screen surface */
SDL_Surface *screen = NULL;
SDL_Surface *title=NULL;
SDL_Surface *gfx=NULL;
SDL_Surface *numbers=NULL;

Mix_Chunk *pop=NULL;
Mix_Chunk *ballbat=NULL;
Mix_Chunk *applause=NULL;
Mix_Chunk *bomb=NULL;
Mix_Music *hiya=NULL;
Mix_Music *rush=NULL;
Mix_Music *alone=NULL;

/* This function draws to the screen; replace this with your own code! */
static void draw()
{
	draw_map();

    /* Make sure everything is displayed on screen */
    SDL_Flip (screen);
    /* Don't run too fast */
    SDL_Delay (1);
}

void draw_tile(int id,int x,int y)
{
    SDL_Rect tileRect;
    SDL_Rect rect;
	
	rect.x=x;
	rect.y=y;
	rect.w=TILEWIDTH;
	rect.h=TILEHEIGHT;
	if(gfx) {
		tileRect.x=id*TILEWIDTH;
		tileRect.y=0;
		tileRect.w=TILEWIDTH;
		tileRect.h=TILEHEIGHT;
		SDL_BlitSurface(gfx,&tileRect,screen,&rect);
	} else {
	    Uint32 color;
	
	    /* Create a black background */
	    color = SDL_MapRGB (screen->format, id&4?255:0, id&2?255:0, id&1?255:0);
		SDL_FillRect(screen,&rect,color);
	}
}

void draw_number(int id,int x,int y)
{
    SDL_Rect tileRect;
    SDL_Rect rect;
	
	rect.x=x;
	rect.y=y;
	rect.w=TILEWIDTH;
	rect.h=TILEHEIGHT;
	if(numbers) {
		tileRect.x=id*TILEWIDTH;
		tileRect.y=0;
		tileRect.w=TILEWIDTH;
		tileRect.h=TILEHEIGHT;
		SDL_BlitSurface(numbers,&tileRect,screen,&rect);
	} else {
	    Uint32 color;
	
	    /* Create a black background */
	    color = SDL_MapRGB (screen->format, id&4?255:0, id&2?255:0, id&1?255:0);
		SDL_FillRect(screen,&rect,color);
	}
}

void play_sound(int id)
{
	static int lastmusic=0;
	
	switch(id) {
	case S_POP:
		Mix_PlayChannel(1,pop,0);
		break;
	case S_BALLBAT:
		Mix_PlayChannel(2,ballbat,0);
		break;
	case S_APPLAUSE:
		Mix_PauseMusic();
		Mix_PlayChannel(3,applause,0);
		SDL_Delay(5000);
		Mix_ResumeMusic();
		break;
	case S_BOMB:
		Mix_PauseMusic();
		Mix_PlayChannel(3,bomb,0);
		SDL_Delay(5000);
		Mix_ResumeMusic();
		break;
	case S_HIYA:
		if(lastmusic!=S_HIYA) {
			Mix_PlayMusic(hiya,1);
		}
		lastmusic=S_HIYA;
		break;
	case S_RUSH:
		if(lastmusic!=S_RUSH) {
			Mix_PlayMusic(rush,1);
		}
		lastmusic=S_RUSH;
		break;
	case S_ALONE:
		if(lastmusic!=S_ALONE) {
			Mix_PlayMusic(alone,1);
		}
		lastmusic=S_ALONE;
		break;
	}
}

int
main (int argc, char *argv[])
{
    char *msg;
    int done;

    /* Initialize SDL */
    if (SDL_Init (SDL_INIT_VIDEO|SDL_INIT_JOYSTICK|SDL_INIT_AUDIO) < 0)
    {
#ifdef _WIN32
        sprintf (msg, "Couldn't initialize SDL: %s\n", SDL_GetError ());
        MessageBox (0, msg, "Error", MB_ICONHAND); 
        free (msg);
#endif
        exit (1);
    }
    atexit (SDL_Quit);

    /* Set 640x480 16-bits video mode */
    screen = SDL_SetVideoMode (480, 272, 16, SDL_SWSURFACE | SDL_DOUBLEBUF);
    if (screen == NULL)
    {
#ifdef _WIN32
        sprintf (msg, "Couldn't set 480x272x16 video mode: %s\n",
          SDL_GetError ());
        MessageBox (0, msg, "Error", MB_ICONHAND); 
        free (msg);
#endif
        exit (2);
    }
    SDL_WM_SetCaption ("Jumping Jack SDL by Dale Wick", NULL);

    gfx=SDL_LoadBMP("gfx.bmp");
    if(gfx) SDL_SetColorKey(gfx, SDL_SRCCOLORKEY, SDL_MapRGB(gfx->format, 0, 255, 0));
    numbers=SDL_LoadBMP("numbers.bmp");
    title=SDL_LoadBMP("title.bmp");

	SDL_Joystick *joy=0;
	if(SDL_NumJoysticks()>0)
	   joy=SDL_JoystickOpen(0);

    if(Mix_OpenAudio(44100,AUDIO_S16SYS, 2, 2048)<0) {
#ifdef _WIN32
        sprintf(msg,"Warning: Couldn't set 44100 Hz 16-bit audio\r\n"
          "- Reason: %s\n",SDL_GetError());
        MessageBox(0,msg,"Warning",MB_ICONHAND);
#endif
    }
    
    //pop=Mix_LoadWAV("ching.WAV");
    ballbat=Mix_LoadWAV("doodloop.WAV");
    applause=Mix_LoadWAV("applause.WAV");
    bomb=Mix_LoadWAV("collide.WAV");
    //hiya=Mix_LoadMUS("hiya.xm");
    rush=Mix_LoadMUS("rush.it");
    alone=Mix_LoadMUS("alone.it");

#if 0
static char *title_screen[]={
	"                              ",
	"                              ",
	"   !!  !  ! !!!  !!!! !!!     ",
	"  !    !  ! !  ! !    !  !    ",
	"   !!  !  ! !!!  !!   !!!     ",
	"     ! !  ! !    !    ! !     ",
	"  !!!   !!  !    !!!! !  !    ",
	"                              ",
	"                              ",
	"    \"\"\" \"  \"  \"\"   \"\"\" \"\"\"\"   ",
	"   \"    \"  \" \"  \" \"    \"      ",
	"   \"    \"  \" \"  \"  \"   \"      ",
	"   \"    \"\"\"\" \"\"\"\"   \"  \"\"\"    ",
	"   \"    \"  \" \"  \"    \" \"      ",
	"    \"\"\" \"  \" \"  \" \"\"\"  \"\"\"\"   ",
	"                              ",
	"                              ",
};

	int t=0;
	for(t=0;t<120;t++) {
		int i,j;
		for(i=0;i<MAPHEIGHT;i++) {
			for(j=0;j<MAPWIDTH;j++) {
				draw_tile(title_screen[i][j]-' ',j*TILEWIDTH,i*TILEHEIGHT);
			}
		}	
	    SDL_Flip (screen);
		SDL_Delay(1);
	}
#else
    SDL_Rect rect;
	
	rect.x=0;
	rect.y=0;
	rect.w=480;
	rect.h=272;
	SDL_BlitSurface(title,0,screen,&rect);

	SDL_Flip(screen);
	
	int start=0;
    done = 0;
//	SDL_Delay(3000);	
    /* Check for events */
	SDL_Event event;
    while (start==0 && SDL_WaitEvent (&event))
    {
        switch (event.type)
        {
        case SDL_KEYDOWN:
			start=1;
            break;
        case SDL_JOYBUTTONDOWN:
			start=1;
			break;
        case SDL_QUIT:
            done = 1;
            break;
        default:
            break;
        }
        if(start || done) break;
    }
#endif

	new_game();
	new_level(1);
    int oldTime=SDL_GetTicks();

	int dir=-1;
    while (!done)
    {
        SDL_Event event;

        /* Check for events */
        while (SDL_PollEvent (&event))
        {
            switch (event.type)
            {
            case SDL_KEYDOWN:
                if(event.key.keysym.sym == SDLK_ESCAPE) done=1;
                if(event.key.keysym.sym == SDLK_SPACE) explode_path();
				if(event.key.keysym.sym == SDLK_LEFT) dir=0;
				if(event.key.keysym.sym == SDLK_UP) dir=1;
				if(event.key.keysym.sym == SDLK_RIGHT) dir=2;
				if(event.key.keysym.sym == SDLK_DOWN) dir=3;
                break;
            case SDL_KEYUP:
				if(event.key.keysym.sym == SDLK_LEFT && dir==0) dir=-1;
				if(event.key.keysym.sym == SDLK_UP && dir==1) dir=-1;
				if(event.key.keysym.sym == SDLK_RIGHT && dir==2) dir=-1;
				if(event.key.keysym.sym == SDLK_DOWN && dir==3) dir=-1;
				break;
            case SDL_JOYBUTTONDOWN:
                if(event.jbutton.button == 10 /* SELECT */) done=1;
                if(event.jbutton.button == 2 /* CROSS */) explode_path();
				if(event.jbutton.button == 7 /* LEFT */) dir=0;
				if(event.jbutton.button == 8 /* UP */) dir=1;
				if(event.jbutton.button == 9 /* RIGHT */) dir=2;
				if(event.jbutton.button == 6 /* DOWN */) dir=3;
				break;
            case SDL_JOYBUTTONUP:
				if(event.jbutton.button == 7 /* LEFT */ && dir==0) dir=-1;
				if(event.jbutton.button == 8 /* UP */ && dir==1) dir=-1;
				if(event.jbutton.button == 9 /* RIGHT */ && dir==2) dir=-1;
				if(event.jbutton.button == 6 /* DOWN */ && dir==3) dir=-1;
				break;
            case SDL_QUIT:
                done = 1;
                break;
            default:
                break;
            }
        }
        if(dir>=0) move_character(dir);

        /* Draw to screen */
		int newTime=SDL_GetTicks();
        next_frame(newTime-oldTime);
        oldTime=SDL_GetTicks();
        draw ();
    }

    return 0;
}
